<div class="mkdocs-ezglossary-summary table">
<table  id="{{ section }}">
    <thead>
        <tr>
            <th>Term</th>
            <th>Definition</th>
            {% if types|length > 0 %}
            <th>References</th>
            {% endif %}
        </tr>
    </thead>
    <tbody>
        {% for term in terms %}
        <tr>
            <td>{% if defs|length > 0 %}
                {% set def = defs[0] %}
                <a href="{{ root }}{{ def.page.url }}#{{ def.target }}"
                   title="{{ def.definition }}">{{term}}</a>
                {% else %}
                {{ term }}
                {% endif %}
            </td>
            <td>{{term}}</td>
            <td>{{ glossary.definition(section, term) | safe }}</td>
            {% if types|length > 0 %}
            <td>
                <ul>
                {% for entry in glossary.get(section, term, 'refs') %}
                    <li>
                        <a href="{{ root }}{{ entry.page.url }}#{{ entry.target }}">{{ entry.page.title }}</a>
                    </li>
                {% endfor %}
                </ul>
            </td>
            {% endif %}
        </tr>
        {% endfor %}
    </tbody>
</table>
</div>
